#include <stdio.h>
#include <gmp.h>
#include "md5.h"
#include "base64.h"

char base64en[64] = 
	"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";

/* 		+0  +1  +2  +3  +4  +5  +6  +7  +8  +9  +A  +B  +C  +D  +E  +F  */
char base64de[0x100] = 
	{	  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
		  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
		  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 62,  0,  0,  0, 63,
		 52, 53, 54, 55, 56, 57, 58, 59, 60, 61,  0,  0,  0,  0,  0,  0,
		  0,  0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14,
		 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25,  0,  0,  0,  0,  0,
		  0, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40,
		 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51,  0,  0,  0,  0,  0,
		  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
		  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
		  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
		  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
		  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
		  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
		  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,
		  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0};

void base64str_to_mpz(mpz_ptr d,const char *str,int len)
{
	int i;
	int slen;
	
	slen = strlen(str);
	slen = (slen > len)? len: slen;
	
	mpz_set_ui(d,0);
	for(i = slen-1; i >= 0; i--){
		mpz_mul_ui(d,d,64);
		mpz_add_ui(d,d,base64de[str[i]]);
	}
}

void mpz_to_base64str(char *buf,int len,mpz_srcptr d_src)
{
	mpz_t d;
	int i;
	
	mpz_init_set(d,d_src);
	for(i=0;(mpz_sgn(d) != 0) && (i < len);i++){
		buf[i] = base64en[mpz_fdiv_q_ui(d,d,64)];
	}
	for(;i<len;i++)
		buf[i] = base64en[0];
	buf[len] = '\0';
	
	mpz_clear(d);
}

void base64encode(char *buf,const unsigned char *source,int len)
{
	int i,j;
	unsigned int tmp;
	unsigned int mask;
	
	for(i = 0;i < len-2;i += 3){
		tmp = 0;
		tmp |= *source++;
		tmp <<= 8;
		tmp |= *source++;
		tmp <<= 8;
		tmp |= *source++;
		for(mask = 0xfc0000,j = 3;j >= 0;j--,mask >>= 6,buf++)
			*buf = base64en[(tmp & mask) >> j*6];
	}
	if(i != len){
		if(i  == len-1){
			tmp = 0;
			tmp |= *source;
			tmp <<= 4;
			mask = 0xfc0;
			*buf++ = base64en[(tmp & mask) >> 6];
			mask >>= 6;
			*buf++ = base64en[(tmp & mask)];
			*buf++ = '=';
			*buf++ = '=';
		}
		else {
			tmp = 0;
			tmp |= *source++;
			tmp <<= 8;
			tmp |= *source;
			tmp <<= 2;
			mask = 0x3f000;
			*buf++ = base64en[(tmp & mask) >> 12];
			mask >>= 6;
			*buf++ = base64en[(tmp & mask) >> 6];
			mask >>= 6;
			*buf++ = base64en[(tmp & mask)];
			*buf++ = '=';
		}
	}
	*buf = '\0';
}

void base64decode(unsigned char *buf,const unsigned char *source,int len)
{
	int i,j;
	unsigned int tmp;
	unsigned int mask;
	
	for(i = 0;i < len;i += 4){
		tmp = 0;
		for(j = 0;j < 4;j++){
			tmp <<= 6;
			tmp |= base64de[*source++];
		}
		*buf++ = tmp >> 16;
		*buf++ = (tmp & 0xff00) >> 8;
		*buf++ = tmp & 0xff;
	}
	*buf = '\0';
}


void triphash(char *dest,const char *keystr,int len)
{
	char md5[16];
	char buf[32];
	
	md5_buffer(keystr,len,md5);
	base64encode(buf,md5,16);
	strncpy(dest,buf,11);
	dest[11] = '\0';
}

